@@ -54,7 +54,13 @@ class Signal:
             
         return audio
     
-    def mixdown(self, sample_rate, byte_width=2, max_amplitude=1):
+    mixdown = realise # TODO consider keeping just 1
+    
+    def play(self, sample_rate=44100, byte_width=2, max_amplitude=1):
+        audio = self.realise(sample_rate)
+        audio.play(byte_width)
+    
+    def export(self, filename, sample_rate=44100, byte_width=2, max_amplitude=1):
         """
         0 < max_amplitude <= 1 implies stretching the amplitudes
         so they would hit absolute value of max_amplitude.
@@ -62,13 +68,9 @@ class Signal:
         as given, unless they exceed 1 in which case we shrink everything proportionally.
         TODO not sure this behaviour is optimal
         """
-        if byte_width != 2:
-            print("Unstable for byte widths other than 2; expect future support.")
-        
         audio = self.realise(sample_rate)
-        audio.push_forward(audio.shift) # TODO what if shift is negative? bug or feature?
-        # in case the top-level signal has non-zero shift
-        return audio.mixdown(byte_width, max_amplitude)
+        audio.to_WAV(filename, byte_width)
+        
     
     #####################
     def num_samples(self, sample_rate):
@@ -110,7 +112,7 @@ class Signal:
         if transform is None:
             return self
         
-        if not isinstance(transform, Transform):
+        if not isinstance(transform, Transform): # TODO change to if isnumber(transform)
             return self._amplitude(transform)
         
         s = self.copy() # TODO is this needed?
@@ -398,13 +400,14 @@ class WhiteNoise(Signal):
 # TODO add grey, pink noise.
 
 
-#### single-pitched signals
+#### simple oscillators
 
 
 class Sine(Signal): # oscillator? pitch? phaser?
     wave = np.sin
     
     def __init__(self, frequency=220, duration=5e3):
+        # TODO add initial phase argument (for completeness; also for potential phase inference)
         super().__init__()
         self.frequency = read_freq(frequency)
         self.duration = duration
@@ -451,12 +454,22 @@ class Raw(Signal):
             if self._key() not in Raw.cache:
                 Raw.cache[self._key()] = audio
         
-    def _key(self):
+    def _key(self): # TODO __key__ ?
         return type(self).__name__ + ":" + str(self.key)
     
+    def resample(self, sample_rate=44100, method="quadratic"):
+        self.audio._resample(sample_rate, method)
+        return self # so the user can type WAV("a.wav").resample(44100)*Transform() etc.
+        ...
+    
+    @property
+    def audio(self):
+        # return Audio object, not ndarray, to override sample raes TODO see if this makes sense overall
+        return Raw.cache[self._key()]#.audio
+    
     def generate(self, sample_rate):
         #return np.copy(self.audio.audio)
-        return Raw.cache[self._key()].audio
+        return self.audio.audio
     """
     TODO
     ####think about this more. here we're copying the audio data,
