@@ -11,7 +11,7 @@ from gensound.curve import Curve, Line, Logistic, Constant
 from gensound.audio import Audio
 from gensound.utils import lambda_to_range, DB_to_Linear, \
                   isnumber, iscallable, \
-                  num_samples, samples_slice
+                  num_samples, samples_slice, sec
 
 __all__ = ["Transform", "Shift", "Extend", "Reverse", "Fade", "CrossFade",
            "Gain", "SineAM", "Limiter", "Mono", "Pan", "Repan", "Downsample",
@@ -211,7 +211,7 @@ class Fade(Transform):
         # TODO I still hear a bump when the playback starts
 
 
-class CrossFade(BiTransform):
+class CrossFade(BiTransform): # TODO rename to XFade?
     def __init__(self, duration):
         L = Fade(is_in=False, duration=duration)
         R = Fade(duration=duration)*Shift(-duration)
@@ -300,7 +300,7 @@ class SineAM(Transform):
         assert isinstance(audio, Audio) # TODO remove this after debug hopefully
         
         sin = np.sin(self.phase + self.frequency * \
-                     np.linspace(0, audio.duration(), audio.length, False) * 2 * np.pi)
+                     np.linspace(0, audio.duration/sec, audio.length, False) * 2 * np.pi)
         audio *= (sin * self.size + (1-self.size))
         # remember [:] is necessary to retain changes
         
@@ -433,6 +433,34 @@ class Repan(Transform):
 
 ####### FILTERS #########
 
+class Stretch(Transform):
+    """ Stretches audio by a certain factor, or to a desired duration,
+    using interpolation.
+    """
+    # TODO parametric stretch!
+    def __init__(self, rate=None, duration=None, method="quadratic"):
+        """ exactly one of factor, duration should be defined.
+        factor = ratio of speed up/slow down. (1=no difference, 2=twice as fast)
+        duration = stretch/shrink to this duration (milliseconds or samples)
+        """
+        self.rate = rate
+        self.duration = duration
+        self.method = method
+    
+    def realise(self, audio):
+        assert (self.rate is None) + (self.duration is None) == 1, "Stretch: exactly one of the arguments rate, duration, must be defined."
+        from gensound.utils import get_interpolation
+        interpolate = get_interpolation(self.method)
+        
+        if self.rate is None:
+            factor = audio.duration / self.duration
+        else:
+            factor = self.rate
+        
+        audio.audio = interpolate(audio.audio, np.arange(0, audio.length-1, factor))
+        
+
+
 class Downsample(Transform):
     """ skips samples. can hear the effects of aliasing.
     suppose factor is 3, then this copies all 3k-th samples into
@@ -482,8 +510,21 @@ class Convolution(Transform):
 '''
 
 class Convolution(Transform):
+    """ Convolves a Signal with given samples (self.response), which may be given
+    either as an Audio object, a WAV filename, or np.ndarray. It is internally
+    stored as the latter, though this may change.
+    
+    If response is mono, each channel of the signal is convolved individually.
+    If the signal is mono, it is convolved with each of the response channels
+    individually, yielding an output with the same number of channels as response.
+    If multi-channel output is undesirable, crop the response audio to one channel
+    before applying this transform.
+    If neither is mono, then the audio and response must have the same number of
+    channels, and they are convolved individually.
+    """
     def __init__(self, response):
         assert isinstance(response, (Audio, np.ndarray, str)) # Audio, direct buffer, or filename
+        # TODO: accept Signal as response?
         assert "scipy" in _supported, "Convolution: SciPy not supported"
         
         if isinstance(response, np.ndarray):
@@ -492,25 +533,33 @@ class Convolution(Transform):
             self.response = response.audio
         else:
             from gensound.signals import WAV
-            self.response = WAV(response).audio.audio # load file directly to Audio
-        # TODO only accepts mono response
+            self.response = WAV(response).audio # load file directly to Audio        
+        
+        # TODO consider converting self.response to Audio
         
         if len(self.response.shape) == 1: # ensure dimensionality. TODO too similar to Audio.ensure_2d!
             self.response.resize((1, self.response.shape[0]))
     
     def realise(self, audio):
-        assert self.response.shape[0] in (audio.audio.shape[0], 1)
+        assert self.response.shape[0] in (audio.audio.shape[0], 1) or audio.is_mono, \
+            "Convolution channel number mismatch: convolved audios must either " \
+            "match in channel number, or at least one must be mono."
         # self.response should be either mono or have the same number of channels as audio
         
         from scipy.signal import convolve, oaconvolve
         
+        # TODO this is too wordy
         # TODO mode="same" possibly should be "full"
-        # TODO How does/should this behave when convolving stereo with stereo?
         if self.response.shape[0] == 1: # apply same mono reverb to all channels
             for i in range(audio.num_channels):
                 audio.audio[i,:] = convolve(audio.audio[i,:], self.response[0,:], mode="same")
-        else:
-            audio.audio[:,:] = convolve(audio.audio[:,:], self.response[:,:], mode="same")
+        elif audio.is_mono: # input signal gains new channels
+            audio.from_mono(self.response.shape[0])
+            for i in range(self.response.shape[0]):
+                audio.audio[i,:] = convolve(audio.audio[i,:], self.response[i,:], mode="same")
+        else: # both audio and response non-mono, same number of channels
+            for i in range(self.response.shape[0]): # "Parallel Stereo"
+                audio.audio[i,:] = convolve(audio.audio[i,:], self.response[i,:], mode="same")
 
 
 class ADSR(Transform):
@@ -528,7 +577,7 @@ class ADSR(Transform):
         env_start = Line(0, 1, self.attack) | Constant(1, self.hold)
         env_start |= Line(1, self.sustain, duration = self.decay)
         length_start = env_start.num_samples(audio.sample_rate)
-        # or maybe as one envelope by extrpolating from audio.duration()?
+        # or maybe as one envelope by extrpolating from audio.duration?
         env_end = Line(self.sustain, 0, duration=self.release)
         length_end = env_end.num_samples(audio.sample_rate)
         
